import React, { forwardRef, useImperativeHandle, useState } from 'react';
import { Image, Linking, Share, TouchableOpacity, View } from 'react-native';
import { Text } from '../../Atoms';
import { styles } from './modal.styles';
import ModalBottom from './ModalBottom';

export const socialUrl = {
  FACEBOOK: 'https://www.facebook.com/sharer/sharer.php?',
  TWITTER: 'https://twitter.com/intent/tweet?',
  WHATSAPP: 'https://wa.me?',
};

async function openUrl(url) {
  return new Promise((resolve, reject) => {
    Linking.canOpenURL(url)
      .then((supported) => {
        if (supported) {
          Linking.openURL(url)
            .then((data) => {
              return resolve(true);
            })
            .catch(() => {
              return reject(false);
            });
        }
      })
      .catch((err) => {
        throw reject(err);
      });
  });
}

async function handlePostToFaceBook(shareLink, postContent) {
  let facebookParameters = [];
  facebookParameters.push('u=' + encodeURI(shareLink));
  facebookParameters.push('quote=' + encodeURI(postContent));
  const url = socialUrl.FACEBOOK + facebookParameters.join('&');
  const openned = await openUrl(url);
  return openned;
}

async function handleTwit(shareLink, tweetContent) {
  let twitterParameters = [];
  twitterParameters.push('url=' + encodeURI(shareLink));
  twitterParameters.push('text=' + encodeURI(tweetContent));
  const url = socialUrl.TWITTER + twitterParameters.join('&');

  const openned = await openUrl(url);
  return openned;
}

async function shareToWa(shareLink, msg) {
  let url = `${socialUrl.WHATSAPP}text=${msg}`;

  const openned = await openUrl(url);
  return openned;
}

async function onShare(content) {
  try {
    const result = await Share.share({
      message: content,
    });
    if (result.action === Share.sharedAction) {
      if (result.activityType) {
        // shared with activity type of result.activityType
      } else {
        // shared
      }
    } else if (result.action === Share.dismissedAction) {
      // dismissed
    }
  } catch (error) {
    // alert(error.message);
    console.log('error', error);
  }
}

const ShareModal = forwardRef(({ ...props }, ref) => {
  const SocialsApp = [
    {
      id: 0,
      name: 'Facebook',
      icon: require('../../../assets/icons/socials/fb.png'),
      handleShare: (url, content) => handlePostToFaceBook(url, content),
    },
    {
      id: 1,
      name: 'Twitter',
      icon: require('../../../assets/icons/socials/twitter.png'),
      handleShare: (url, content) => handleTwit(url, content),
    },
    {
      id: 2,
      name: 'Whatsapp',
      icon: require('../../../assets/icons/socials/wa.png'),
      handleShare: (url, content) => shareToWa(url, content),
    },
    {
      id: 3,
      name: 'Lainnya',
      icon: require('../../../assets/icons/socials/more.png'),
      handleShare: (url, content) => onShare(content),
    },
  ];

  const [showModalShare, setShowModalShare] = useState({
    title: 'Bagikan Produk Melalui',
    isVisible: false,
    sharedUrl: 'https://hidoop.link/0pwGb6mnOo6',
    content: '',
  });

  useImperativeHandle(ref, () => ({
    _showModal: (data) => showModal(data),
    _closeModal: () => closeModal(),
  }));

  return (
    <ModalBottom
      title={showModalShare?.title}
      isVisible={showModalShare?.isVisible}
      onBackdropPress={closeModal}
      onBackButtonPress={closeModal}
      body={
        <View style={styles.modalBody}>
          {SocialsApp.map((i) => {
            const _handleShare = async () => {
              i.handleShare(showModalShare?.sharedUrl, showModalShare?.content);
            };

            return (
              <TouchableOpacity
                key={i.id}
                onPress={_handleShare}
                style={styles.itemSocial}
              >
                <Image source={i.icon} style={styles.imgSocial} />
                <Text text={i?.name} size="small" />
              </TouchableOpacity>
            );
          })}
        </View>
      }
    />
  );

  function showModal(data = {}) {
    setShowModalShare({
      ...showModalShare,
      ...data,
      isVisible: true,
    });
  }

  function closeModal() {
    setShowModalShare({
      isVisible: false,
      title: 'Bagikan Produk Melalui',
      url: '',
      content: '',
    });
  }
});

export default ShareModal;
